@@ -1,60 +1,26 @@
 import json
-import sys
 import time
-import uuid
-from functools import cmp_to_key, reduce
+from functools import reduce
 from typing import List, Optional, Tuple
 
-import yaml
 from botocore.client import BaseClient
 
 from dstack import version
-from dstack.backend.aws import jobs, logs
+from dstack.backend.base.compute import choose_instance_type
 from dstack.core.instance import InstanceType
 from dstack.core.job import Job, JobStatus, Requirements
 from dstack.core.repo import RepoAddress
 from dstack.core.request import RequestHead, RequestStatus
-from dstack.core.runners import Gpu, Resources, Runner
+from dstack.core.runners import Gpu, Resources
 
 CREATE_INSTANCE_RETRY_RATE_SECS = 3
 
 
-def _serialize_runner(runner: Runner) -> dict:
-    resources = {
-        "cpus": runner.resources.cpus,
-        "memory_mib": runner.resources.memory_mib,
-        "gpus": [
-            {
-                "name": gpu.name,
-                "memory_mib": gpu.memory_mib,
-            }
-            for gpu in (runner.resources.gpus or [])
-        ],
-        "interruptible": runner.resources.interruptible is True,
-        "local": runner.resources.local is True,
-    }
-    data = {
-        "runner_id": runner.runner_id,
-        "request_id": runner.request_id,
-        "resources": resources,
-        "job": runner.job.serialize(),
-    }
-    return data
-
-
-def _unserialize_runner(data: dict) -> Runner:
-    return Runner(
-        data["runner_id"],
-        data.get("request_id"),
-        Resources(
-            data["resources"]["cpus"],
-            data["resources"]["memory_mib"],
-            [Gpu(g["name"], g["memory_mib"]) for g in data["resources"]["gpus"]],
-            data["resources"]["interruptible"] is True,
-            data["resources"].get("local") is True,
-        ),
-        Job.unserialize(data["job"]),
-    )
+def get_instance_type(
+    ec2_client: BaseClient, requirements: Optional[Requirements]
+) -> Optional[InstanceType]:
+    instance_types = _get_instance_types(ec2_client)
+    return choose_instance_type(instance_types, requirements)
 
 
 def _get_instance_types(ec2_client: BaseClient) -> List[InstanceType]:
@@ -68,7 +34,7 @@ def _get_instance_types(ec2_client: BaseClient) -> List[InstanceType]:
             Filters=[
                 {
                     "Name": "instance-type",
-                    "Values": ["c5.*", "m5.*", "p2.*", "p3.*", "p4d.*", "p4de.*"],
+                    "Values": ["t2.small", "c5.*", "m5.*", "p2.*", "p3.*", "p4d.*", "p4de.*"],
                 },
             ],
             **kwargs,
@@ -76,7 +42,8 @@ def _get_instance_types(ec2_client: BaseClient) -> List[InstanceType]:
         for instance_type in response["InstanceTypes"]:
             gpus = (
                 [
-                    [Gpu(gpu["Name"], gpu["MemoryInfo"]["SizeInMiB"])] * gpu["Count"]
+                    [Gpu(name=gpu["Name"], memory_mib=gpu["MemoryInfo"]["SizeInMiB"])]
+                    * gpu["Count"]
                     for gpu in instance_type["GpuInfo"]["Gpus"]
                 ]
                 if instance_type.get("GpuInfo") and instance_type["GpuInfo"].get("Gpus")
@@ -84,126 +51,20 @@ def _get_instance_types(ec2_client: BaseClient) -> List[InstanceType]:
             )
             instance_types.append(
                 InstanceType(
-                    instance_type["InstanceType"],
-                    Resources(
-                        instance_type["VCpuInfo"]["DefaultVCpus"],
-                        instance_type["MemoryInfo"]["SizeInMiB"],
-                        reduce(list.__add__, gpus) if gpus else [],
-                        "spot" in instance_type["SupportedUsageClasses"],
-                        False,
+                    instance_name=instance_type["InstanceType"],
+                    resources=Resources(
+                        cpus=instance_type["VCpuInfo"]["DefaultVCpus"],
+                        memory_mib=instance_type["MemoryInfo"]["SizeInMiB"],
+                        gpus=reduce(list.__add__, gpus) if gpus else [],
+                        interruptible="spot" in instance_type["SupportedUsageClasses"],
+                        local=False,
                     ),
                 )
             )
-
-    def compare(i1, i2):
-        r1_gpu_total_memory_mib = sum(map(lambda g: g.memory_mib, i1.resources.gpus or []))
-        r2_gpu_total_memory_mib = sum(map(lambda g: g.memory_mib, i2.resources.gpus or []))
-        if r1_gpu_total_memory_mib < r2_gpu_total_memory_mib:
-            return -1
-        elif r1_gpu_total_memory_mib > r2_gpu_total_memory_mib:
-            return 1
-        if i1.resources.cpus < i2.resources.cpus:
-            return -1
-        elif i1.resources.cpus > i2.resources.cpus:
-            return 1
-        if i1.resources.memory_mib < i2.resources.memory_mib:
-            return -1
-        elif i1.resources.memory_mib > i2.resources.memory_mib:
-            return 1
-        return 0
-
-    return sorted(instance_types, key=cmp_to_key(compare))
+    return instance_types
 
 
-def _matches(resources: Resources, requirements: Optional[Requirements]) -> bool:
-    if not requirements:
-        return True
-    if requirements.cpus and requirements.cpus > resources.cpus:
-        return False
-    if requirements.memory_mib and requirements.memory_mib > resources.memory_mib:
-        return False
-    if requirements.gpus:
-        gpu_count = requirements.gpus.count or 1
-        if gpu_count > len(resources.gpus or []):
-            return False
-        if requirements.gpus.name and gpu_count > len(
-            list(filter(lambda gpu: gpu.name == requirements.gpus.name, resources.gpus or []))
-        ):
-            return False
-        if requirements.gpus.memory_mib and gpu_count > len(
-            list(
-                filter(
-                    lambda gpu: gpu.memory_mib >= requirements.gpus.memory_mib,
-                    resources.gpus or [],
-                )
-            )
-        ):
-            return False
-        if requirements.interruptible and not resources.interruptible:
-            return False
-    return True
-
-
-def _get_instance_type(
-    ec2_client: BaseClient, requirements: Optional[Requirements]
-) -> Optional[InstanceType]:
-    instance_types = _get_instance_types(ec2_client)
-
-    instance_type = next(
-        (
-            instance_type
-            for instance_type in instance_types
-            if _matches(instance_type.resources, requirements)
-        ),
-        None,
-    )
-    return (
-        InstanceType(
-            instance_type.instance_name,
-            Resources(
-                instance_type.resources.cpus,
-                instance_type.resources.memory_mib,
-                instance_type.resources.gpus,
-                requirements and requirements.interruptible,
-                False,
-            ),
-        )
-        if instance_type
-        else None
-    )
-
-
-def _create_runner(
-    logs_client: BaseClient, s3_client: BaseClient, bucket_name: str, runner: Runner
-):
-    key = f"runners/{runner.runner_id}.yaml"
-    metadata = {}
-    if runner.job.status == JobStatus.STOPPING:
-        metadata["status"] = "stopping"
-    s3_client.put_object(
-        Body=yaml.dump(_serialize_runner(runner)),
-        Bucket=bucket_name,
-        Key=key,
-        Metadata=metadata,
-    )
-    log_group_name = f"/dstack/runners/{bucket_name}"
-    logs.create_log_group_if_not_exists(logs_client, bucket_name, log_group_name)
-
-
-def _update_runner(s3_client: BaseClient, bucket_name: str, runner: Runner):
-    key = f"runners/{runner.runner_id}.yaml"
-    metadata = {}
-    if runner.job.status == JobStatus.STOPPING:
-        metadata["status"] = "stopping"
-    s3_client.put_object(
-        Body=yaml.dump(_serialize_runner(runner)),
-        Bucket=bucket_name,
-        Key=key,
-        Metadata=metadata,
-    )
-
-
-def get_security_group_id(ec2_client: BaseClient, bucket_name: str, subnet_id: Optional[str]):
+def _get_security_group_id(ec2_client: BaseClient, bucket_name: str, subnet_id: Optional[str]):
     _subnet_postfix = (subnet_id.replace("-", "_") + "_") if subnet_id else ""
     security_group_name = (
         "dstack_security_group_" + _subnet_postfix + bucket_name.replace("-", "_").lower()
@@ -412,7 +273,7 @@ def role_name(iam_client: BaseClient, bucket_name: str) -> str:
     return _role_name
 
 
-def instance_profile_arn(iam_client: BaseClient, bucket_name: str) -> str:
+def _get_instance_profile_arn(iam_client: BaseClient, bucket_name: str) -> str:
     _role_name = role_name(iam_client, bucket_name)
     try:
         response = iam_client.get_instance_profile(InstanceProfileName=_role_name)
@@ -508,12 +369,12 @@ def _run_instance(
                 "AssociatePublicIpAddress": True,
                 "DeviceIndex": 0,
                 "SubnetId": subnet_id,
-                "Groups": [get_security_group_id(ec2_client, bucket_name, subnet_id)],
+                "Groups": [_get_security_group_id(ec2_client, bucket_name, subnet_id)],
             },
         ]
     else:
         launch_specification["SecurityGroupIds"] = [
-            get_security_group_id(ec2_client, bucket_name, subnet_id)
+            _get_security_group_id(ec2_client, bucket_name, subnet_id)
         ]
     tags = [
         {"Key": "owner", "Value": "dstack"},
@@ -539,7 +400,7 @@ def _run_instance(
         MinCount=1,
         MaxCount=1,
         IamInstanceProfile={
-            "Arn": instance_profile_arn(iam_client, bucket_name),
+            "Arn": _get_instance_profile_arn(iam_client, bucket_name),
         },
         UserData=_user_data(bucket_name, region_name, runner_id, instance_type.resources),
         TagSpecifications=[
@@ -558,7 +419,7 @@ def _run_instance(
     return request_id
 
 
-def _run_instance_retry(
+def run_instance_retry(
     ec2_client: BaseClient,
     iam_client: BaseClient,
     bucket_name: str,
@@ -592,7 +453,7 @@ def _run_instance_retry(
         ):
             if attempts > 0:
                 time.sleep(CREATE_INSTANCE_RETRY_RATE_SECS)
-                return _run_instance_retry(
+                return run_instance_retry(
                     ec2_client,
                     iam_client,
                     bucket_name,
@@ -611,73 +472,7 @@ def _run_instance_retry(
             raise e
 
 
-def run_job(
-    logs_client: BaseClient,
-    ec2_client: BaseClient,
-    iam_client: BaseClient,
-    s3_client: BaseClient,
-    bucket_name: str,
-    region_name,
-    subnet_id: Optional[str],
-    job: Job,
-):
-    if job.status != JobStatus.SUBMITTED:
-        raise Exception("Can't create a request for a job which status is not SUBMITTED")
-
-    runner = None
-    try:
-        job.runner_id = uuid.uuid4().hex
-        jobs.update_job(s3_client, bucket_name, job)
-        instance_type = _get_instance_type(ec2_client, job.requirements)
-        if instance_type is None:
-            job.status = JobStatus.FAILED
-            jobs.update_job(s3_client, bucket_name, job)
-            sys.exit(f"No instance type matching requirements.")
-
-        runner = Runner(job.runner_id, None, instance_type.resources, job)
-        _create_runner(logs_client, s3_client, bucket_name, runner)
-        runner.request_id = _run_instance_retry(
-            ec2_client,
-            iam_client,
-            bucket_name,
-            region_name,
-            subnet_id,
-            job.runner_id,
-            instance_type,
-            job.local_repo_user_name,
-            job.local_repo_user_email,
-            job.repo_address,
-        )
-        _update_runner(s3_client, bucket_name, runner)
-    except Exception as e:
-        job.status = JobStatus.FAILED
-        job.request_id = runner.request_id if runner else None
-        jobs.update_job(s3_client, bucket_name, job)
-        raise e
-
-
-def _delete_runner(s3_client: BaseClient, bucket_name: str, runner: Runner):
-    key = f"runners/{runner.runner_id}.yaml"
-    s3_client.delete_object(Bucket=bucket_name, Key=key)
-
-
-def _get_runner(s3_client: BaseClient, bucket_name: str, runner_id: str) -> Optional[Runner]:
-    key = f"runners/{runner_id}.yaml"
-    try:
-        obj = s3_client.get_object(Bucket=bucket_name, Key=key)
-        return _unserialize_runner(yaml.load(obj["Body"].read().decode("utf-8"), yaml.FullLoader))
-    except Exception as e:
-        if (
-            hasattr(e, "response")
-            and e.response.get("Error")
-            and e.response["Error"].get("Code") == "NoSuchKey"
-        ):
-            return None
-        else:
-            raise e
-
-
-def _cancel_spot_request(ec2_client: BaseClient, request_id: str):
+def cancel_spot_request(ec2_client: BaseClient, request_id: str):
     ec2_client.cancel_spot_instance_requests(SpotInstanceRequestIds=[request_id])
     response = ec2_client.describe_instances(
         Filters=[
@@ -690,7 +485,7 @@ def _cancel_spot_request(ec2_client: BaseClient, request_id: str):
         )
 
 
-def _terminate_instance(ec2_client: BaseClient, request_id: str):
+def terminate_instance(ec2_client: BaseClient, request_id: str):
     try:
         ec2_client.terminate_instances(InstanceIds=[request_id])
     except Exception as e:
@@ -706,202 +501,114 @@ def _terminate_instance(ec2_client: BaseClient, request_id: str):
 
 def get_request_head(
     ec2_client: BaseClient,
-    s3_client: BaseClient,
-    bucket_name: str,
     job: Job,
-    runner: Optional[Runner] = None,
+    request_id: Optional[str],
 ) -> RequestHead:
-    _local = job.requirements and job.requirements.local
     interruptible = job.requirements and job.requirements.interruptible
-    request_id = None
-    if job.request_id:
-        request_id = job.request_id
-    elif runner and runner.request_id:
-        request_id = runner.request_id
-    elif not runner:
-        runner = _get_runner(s3_client, bucket_name, job.runner_id)
-        if runner:
-            request_id = runner.request_id
-    if request_id:
-        """
-        if _local:
-            _running = local.is_running(request_id)
-            return RequestHead(job.job_id, RequestStatus.RUNNING if _running else RequestStatus.TERMINATED, None)
-        el ivan
-        """
-        if interruptible:
-            try:
-                response = ec2_client.describe_spot_instance_requests(
-                    SpotInstanceRequestIds=[request_id]
-                )
-                if response.get("SpotInstanceRequests"):
-                    status = response["SpotInstanceRequests"][0]["Status"]
-                    if status["Code"] in [
-                        "fulfilled",
-                        "request-canceled-and-instance-running",
-                    ]:
-                        request_status = RequestStatus.RUNNING
-                    elif status["Code"] in [
-                        "not-scheduled-yet",
-                        "pending-evaluation",
-                        "pending-fulfillment",
-                    ]:
-                        request_status = RequestStatus.PENDING
-                    elif status["Code"] in [
-                        "capacity-not-available",
-                        "instance-stopped-no-capacity",
-                        "instance-terminated-by-price",
-                        "instance-stopped-by-price",
-                        "instance-terminated-no-capacity",
-                        "limit-exceeded",
-                        "price-too-low",
-                    ]:
-                        request_status = RequestStatus.NO_CAPACITY
-                    elif status["Code"] in [
-                        "instance-terminated-by-user",
-                        "instance-stopped-by-user",
-                        "canceled-before-fulfillment",
-                        "instance-terminated-by-schedule",
-                        "instance-terminated-by-service",
-                        "spot-instance-terminated-by-user",
-                        "marked-for-stop",
-                        "marked-for-termination",
-                    ]:
-                        request_status = RequestStatus.TERMINATED
-                    else:
-                        raise Exception(
-                            f"Unsupported EC2 spot instance request status code: {status['Code']}"
-                        )
-                    return RequestHead(job.job_id, request_status, status.get("Message"))
-                else:
-                    return RequestHead(job.job_id, RequestStatus.TERMINATED, None)
-            except Exception as e:
-                if (
-                    hasattr(e, "response")
-                    and e.response.get("Error")
-                    and e.response["Error"].get("Code") == "InvalidSpotInstanceRequestID.NotFound"
-                ):
-                    return RequestHead(
-                        job.job_id,
-                        RequestStatus.TERMINATED,
-                        e.response["Error"].get("Message"),
-                    )
-                else:
-                    raise e
-        else:
-            try:
-                response = ec2_client.describe_instances(InstanceIds=[request_id])
-                if response.get("Reservations") and response["Reservations"][0].get("Instances"):
-                    state = response["Reservations"][0]["Instances"][0]["State"]
-                    if state["Name"] in ["running"]:
-                        request_status = RequestStatus.RUNNING
-                    elif state["Name"] in ["pending"]:
-                        request_status = RequestStatus.PENDING
-                    elif state["Name"] in [
-                        "shutting-down",
-                        "terminated",
-                        "stopping",
-                        "stopped",
-                    ]:
-                        request_status = RequestStatus.TERMINATED
-                    else:
-                        raise Exception(f"Unsupported EC2 instance state name: {state['Name']}")
-                    return RequestHead(job.job_id, request_status, None)
-                else:
-                    return RequestHead(job.job_id, RequestStatus.TERMINATED, None)
-            except Exception as e:
-                if (
-                    hasattr(e, "response")
-                    and e.response.get("Error")
-                    and e.response["Error"].get("Code") == "InvalidInstanceID.NotFound"
-                ):
-                    return RequestHead(
-                        job.job_id,
-                        RequestStatus.TERMINATED,
-                        e.response["Error"].get("Message"),
-                    )
-                else:
-                    raise e
-    else:
+    if request_id is None:
         message = (
             "The spot instance request ID is not specified"
             if interruptible
             else "The instance ID is not specified"
         )
-        return RequestHead(job.job_id, RequestStatus.TERMINATED, message)
+        return RequestHead(job_id=job.job_id, status=RequestStatus.TERMINATED, message=message)
 
-
-def _stop_runner(ec2_client: BaseClient, s3_client: BaseClient, bucket_name: str, runner: Runner):
-    if runner.request_id:
-        if runner.resources.local:
-            pass
-            # local.stop_process(runner.request_id) IVAN
-        elif runner.resources.interruptible:
-            _cancel_spot_request(ec2_client, runner.request_id)
-        else:
-            _terminate_instance(ec2_client, runner.request_id)
-    _delete_runner(s3_client, bucket_name, runner)
-
-
-def stop_job(
-    ec2_client: BaseClient,
-    s3_client: BaseClient,
-    bucket_name: str,
-    repo_address: RepoAddress,
-    job_id: str,
-    abort: bool,
-):
-    job_head = jobs.list_job_head(s3_client, bucket_name, repo_address, job_id)
-    job = jobs.get_job(s3_client, bucket_name, repo_address, job_id)
-    runner = _get_runner(s3_client, bucket_name, job.runner_id) if job else None
-    request_status = (
-        get_request_head(ec2_client, s3_client, bucket_name, job, runner).status
-        if job
-        else RequestStatus.TERMINATED
-    )
-    if (
-        job_head
-        and job_head.status.is_unfinished()
-        or job
-        and job.status.is_unfinished()
-        or runner
-        and runner.job.status.is_unfinished()
-        or request_status != RequestStatus.TERMINATED
-    ):
-        if abort:
-            new_status = JobStatus.ABORTED
-        elif (
-            not job_head
-            or job_head.status in [JobStatus.SUBMITTED, JobStatus.DOWNLOADING]
-            or not job
-            or job.status in [JobStatus.SUBMITTED, JobStatus.DOWNLOADING]
-            or request_status == RequestStatus.TERMINATED
-            or not runner
-        ):
-            new_status = JobStatus.STOPPED
-        elif (
-            job_head
-            and job_head.status != JobStatus.UPLOADING
-            or job
-            and job.status != JobStatus.UPLOADING
-        ):
-            new_status = JobStatus.STOPPING
-        else:
-            new_status = None
-        if new_status:
-            if runner and runner.job.status.is_unfinished() and runner.job.status != new_status:
-                if new_status.is_finished():
-                    _stop_runner(ec2_client, s3_client, bucket_name, runner)
+    if interruptible:
+        try:
+            response = ec2_client.describe_spot_instance_requests(
+                SpotInstanceRequestIds=[request_id]
+            )
+            if response.get("SpotInstanceRequests"):
+                status = response["SpotInstanceRequests"][0]["Status"]
+                if status["Code"] in [
+                    "fulfilled",
+                    "request-canceled-and-instance-running",
+                ]:
+                    request_status = RequestStatus.RUNNING
+                elif status["Code"] in [
+                    "not-scheduled-yet",
+                    "pending-evaluation",
+                    "pending-fulfillment",
+                ]:
+                    request_status = RequestStatus.PENDING
+                elif status["Code"] in [
+                    "capacity-not-available",
+                    "instance-stopped-no-capacity",
+                    "instance-terminated-by-price",
+                    "instance-stopped-by-price",
+                    "instance-terminated-no-capacity",
+                    "limit-exceeded",
+                    "price-too-low",
+                ]:
+                    request_status = RequestStatus.NO_CAPACITY
+                elif status["Code"] in [
+                    "instance-terminated-by-user",
+                    "instance-stopped-by-user",
+                    "canceled-before-fulfillment",
+                    "instance-terminated-by-schedule",
+                    "instance-terminated-by-service",
+                    "spot-instance-terminated-by-user",
+                    "marked-for-stop",
+                    "marked-for-termination",
+                ]:
+                    request_status = RequestStatus.TERMINATED
                 else:
-                    runner.job.status = new_status
-                    _update_runner(s3_client, bucket_name, runner)
+                    raise Exception(
+                        f"Unsupported EC2 spot instance request status code: {status['Code']}"
+                    )
+                return RequestHead(
+                    job_id=job.job_id, status=request_status, message=status.get("Message")
+                )
+            else:
+                return RequestHead(
+                    job_id=job.job_id, status=RequestStatus.TERMINATED, message=None
+                )
+        except Exception as e:
             if (
-                job_head
-                and job_head.status.is_unfinished()
-                and job_head.status != new_status
-                or job
-                and job.status.is_unfinished()
-                and job.status != new_status
+                hasattr(e, "response")
+                and e.response.get("Error")
+                and e.response["Error"].get("Code") == "InvalidSpotInstanceRequestID.NotFound"
             ):
-                job.status = new_status
-                jobs.update_job(s3_client, bucket_name, job)
+                return RequestHead(
+                    job_id=job.job_id,
+                    status=RequestStatus.TERMINATED,
+                    message=e.response["Error"].get("Message"),
+                )
+            else:
+                raise e
+    else:
+        try:
+            response = ec2_client.describe_instances(InstanceIds=[request_id])
+            if response.get("Reservations") and response["Reservations"][0].get("Instances"):
+                state = response["Reservations"][0]["Instances"][0]["State"]
+                if state["Name"] in ["running"]:
+                    request_status = RequestStatus.RUNNING
+                elif state["Name"] in ["pending"]:
+                    request_status = RequestStatus.PENDING
+                elif state["Name"] in [
+                    "shutting-down",
+                    "terminated",
+                    "stopping",
+                    "stopped",
+                ]:
+                    request_status = RequestStatus.TERMINATED
+                else:
+                    raise Exception(f"Unsupported EC2 instance state name: {state['Name']}")
+                return RequestHead(job_id=job.job_id, status=request_status, message=None)
+            else:
+                return RequestHead(
+                    job_id=job.job_id, status=RequestStatus.TERMINATED, message=None
+                )
+        except Exception as e:
+            if (
+                hasattr(e, "response")
+                and e.response.get("Error")
+                and e.response["Error"].get("Code") == "InvalidInstanceID.NotFound"
+            ):
+                return RequestHead(
+                    job_id=job.job_id,
+                    status=RequestStatus.TERMINATED,
+                    message=e.response["Error"].get("Message"),
+                )
+            else:
+                raise e
