@@ -1,5 +1,6 @@
-from abc import ABC
+from abc import ABC, abstractmethod
 from pathlib import Path
+from typing import Dict, Optional
 
 import yaml
 
@@ -13,23 +14,37 @@ def get_dstack_dir():
 
 
 class BackendConfig(ABC):
-    NAME = ""
 
-    _configured = False
+    credentials: Optional[Dict] = None
+
+    @abstractmethod
+    def save(self, path: Path = get_config_path()):
+        pass
+
+    @abstractmethod
+    def load(self, path: Path = get_config_path()):
+        pass
+
+
+class Configurator(ABC):
+    NAME = ""
 
     @property
     def name(self):
         return self.NAME or ""
 
-    def save(self, path: Path = get_config_path()):
+    @abstractmethod
+    def configure_cli(self):
         pass
 
-    def load(self, path: Path = get_config_path()):
-        ...
+    @abstractmethod
+    def configure_hub(self, data: Dict):
+        pass
 
-    def configure(self):
-        ...
+    @abstractmethod
+    def get_config(self, data: Dict) -> BackendConfig:
+        pass
 
-    @property
-    def configured(self):
-        return self._configured
+    @abstractmethod
+    def parse_args(self, args: list = []):
+        pass
