@@ -22,6 +22,9 @@ from dstack.core.job import (
 from dstack.core.repo import RepoAddress, RepoData
 from dstack.utils.common import _quoted
 
+DEFAULT_CPU = 2
+DEFAULT_MEM = "8GB"
+
 
 def _str_to_mib(s: str) -> int:
     ns = s.replace(" ", "").lower()
@@ -187,43 +190,34 @@ class Provider:
             env = self.provider_data.get("env") or []
             env.extend(args.env)
             self.provider_data["env"] = env
-        if (
-            args.cpu
-            or args.memory
-            or args.gpu
-            or args.gpu_name
-            or args.gpu_memory
-            or args.shm_size
-            or args.interruptible
-            or args.local
-        ):
-            resources = self.provider_data.get("resources") or {}
-            self.provider_data["resources"] = resources
-            if args.cpu:
-                resources["cpu"] = args.cpu
-            if args.memory:
-                resources["memory"] = args.memory
-            if args.gpu or args.gpu_name or args.gpu_memory:
-                gpu = (
-                    self.provider_data["resources"].get("gpu") or {}
-                    if self.provider_data.get("resources")
-                    else {}
-                )
-                if type(gpu) is int:
-                    gpu = {"count": gpu}
-                resources["gpu"] = gpu
-                if args.gpu:
-                    gpu["count"] = args.gpu
-                if args.gpu_memory:
-                    gpu["memory"] = args.gpu_memory
-                if args.gpu_name:
-                    gpu["name"] = args.gpu_name
-            if args.shm_size:
-                resources["shm_size"] = args.shm_size
-            if args.interruptible:
-                resources["interruptible"] = True
-            if args.local:
-                resources["local"] = True
+
+        resources = self.provider_data.get("resources") or {}
+        self.provider_data["resources"] = resources
+        if args.cpu:
+            resources["cpu"] = args.cpu
+        if args.memory:
+            resources["memory"] = args.memory
+        if args.gpu or args.gpu_name or args.gpu_memory:
+            gpu = (
+                self.provider_data["resources"].get("gpu") or {}
+                if self.provider_data.get("resources")
+                else {}
+            )
+            if type(gpu) is int:
+                gpu = {"count": gpu}
+            resources["gpu"] = gpu
+            if args.gpu:
+                gpu["count"] = args.gpu
+            if args.gpu_memory:
+                gpu["memory"] = args.gpu_memory
+            if args.gpu_name:
+                gpu["name"] = args.gpu_name
+        if args.shm_size:
+            resources["shm_size"] = args.shm_size
+        if args.interruptible:
+            resources["interruptible"] = True
+        if args.local:
+            resources["local"] = True
         if unknown_args:
             self.provider_data["run_args"] = unknown_args
 
@@ -237,36 +231,35 @@ class Provider:
         repo_data = load_repo_data()
         # [TODO] Handle master job
         jobs = []
-        counter = []
-        for job_spec in job_specs:
+        for i, job_spec in enumerate(job_specs):
             submitted_at = int(round(time.time() * 1000))
             job = Job(
-                None,
-                repo_data,
-                self.run_name,
-                self.workflow_name or None,
-                self.provider_name,
-                repo_data.local_repo_user_name,
-                repo_data.local_repo_user_email,
-                JobStatus.SUBMITTED,
-                submitted_at,
-                job_spec.image_name,
-                job_spec.commands,
-                job_spec.env,
-                job_spec.working_dir,
-                job_spec.artifact_specs,
-                job_spec.port_count,
-                None,
-                None,
-                job_spec.requirements,
-                self.dep_specs,
-                job_spec.master_job,
-                job_spec.app_specs,
-                None,
-                None,
-                tag_name,
+                job_id=f"{self.run_name},{self.workflow_name or ''},{i}",
+                repo_data=repo_data,
+                run_name=self.run_name,
+                workflow_name=self.workflow_name or None,
+                provider_name=self.provider_name,
+                local_repo_user_name=repo_data.local_repo_user_name,
+                local_repo_user_email=repo_data.local_repo_user_email,
+                status=JobStatus.SUBMITTED,
+                submitted_at=submitted_at,
+                image_name=job_spec.image_name,
+                commands=job_spec.commands,
+                env=job_spec.env,
+                working_dir=job_spec.working_dir,
+                artifact_specs=job_spec.artifact_specs,
+                port_count=job_spec.port_count,
+                ports=None,
+                host_name=None,
+                requirements=job_spec.requirements,
+                dep_specs=self.dep_specs,
+                master_job=job_spec.master_job,
+                app_specs=job_spec.app_specs,
+                runner_id=None,
+                request_id=None,
+                tag_name=tag_name,
             )
-            backend.submit_job(job, counter)
+            backend.submit_job(job)
             jobs.append(job)
         if tag_name:
             backend.add_tag_from_run(repo_data, tag_name, self.run_name, jobs)
@@ -295,10 +288,11 @@ class Provider:
             return text
 
         if isinstance(artifact, str):
-            return ArtifactSpec(remove_prefix(artifact, "./"), False)
+            return ArtifactSpec(artifact_path=remove_prefix(artifact, "./"), mount=False)
         else:
             return ArtifactSpec(
-                remove_prefix(artifact["path"], "./"), artifact.get("mount") is True
+                artifact_path=remove_prefix(artifact["path"], "./"),
+                mount=artifact.get("mount") is True,
             )
 
     @staticmethod
@@ -322,7 +316,12 @@ class Provider:
                 return Provider._workflow_dep(backend, repo_data, t[0], mount)
         elif len(t) == 3:
             # This doesn't allow to refer to projects from other repos
-            repo_address = RepoAddress(repo_data.repo_host_name, repo_data.repo_port, t[0], t[1])
+            repo_address = RepoAddress(
+                repo_host_name=repo_data.repo_host_name,
+                repo_port=repo_data.repo_port,
+                repo_user_name=t[0],
+                repo_name=t[1],
+            )
             if tag_dep:
                 return Provider._tag_dep(backend, repo_address, t[2], mount)
             else:
@@ -336,7 +335,7 @@ class Provider:
     ) -> DepSpec:
         tag_head = backend.get_tag_head(repo_address, tag_name)
         if tag_head:
-            return DepSpec(repo_address, tag_head.run_name, mount)
+            return DepSpec(repo_address=repo_address, run_name=tag_head.run_name, mount=mount)
         else:
             sys.exit(f"Cannot find the tag '{tag_name}' in the '{repo_address.path()}' repo")
 
@@ -361,7 +360,7 @@ class Provider:
             None,
         )
         if run_name:
-            return DepSpec(repo_address, run_name, mount)
+            return DepSpec(repo_address=repo_address, run_name=run_name, mount=mount)
         else:
             sys.exit(
                 f"Cannot find any successful workflow with the name '{workflow_name}' "
@@ -388,58 +387,47 @@ class Provider:
         else:
             return v
 
-    def _resources(self) -> Optional[Requirements]:
-        if self.provider_data.get("resources"):
-            resources = Requirements()
-            if self.provider_data["resources"].get("cpu"):
-                if not str(self.provider_data["resources"]["cpu"]).isnumeric():
-                    sys.exit("resources.cpu should be an integer")
-                cpu = int(self.provider_data["resources"]["cpu"])
-                if cpu > 0:
-                    resources.cpus = cpu
-            if self.provider_data["resources"].get("memory"):
-                resources.memory_mib = _str_to_mib(self.provider_data["resources"]["memory"])
-            gpu = self.provider_data["resources"].get("gpu")
-            if gpu:
-                if str(gpu).isnumeric():
-                    gpu = int(self.provider_data["resources"]["gpu"])
-                    if gpu > 0:
-                        resources.gpus = GpusRequirements(gpu)
-                else:
-                    gpu_count = 0
-                    gpu_name = None
-                    if str(gpu.get("count")).isnumeric():
-                        gpu_count = int(gpu.get("count"))
-                    if gpu.get("name"):
-                        gpu_name = gpu.get("name")
-                        if not gpu_count:
-                            gpu_count = 1
-                    if gpu_count:
-                        resources.gpus = GpusRequirements(gpu_count, name=gpu_name)
-            for resource_name in self.provider_data["resources"]:
-                if resource_name.endswith("/gpu") and len(resource_name) > 4:
-                    if not str(self.provider_data["resources"][resource_name]).isnumeric():
-                        sys.exit(f"resources.'{resource_name}' should be an integer")
-                    gpu = int(self.provider_data["resources"][resource_name])
-                    if gpu > 0:
-                        resources.gpus = GpusRequirements(gpu, name=resource_name[:-4])
-            if self.provider_data["resources"].get("shm_size"):
-                resources.shm_size_mib = _str_to_mib(self.provider_data["resources"]["shm_size"])
-            if self.provider_data["resources"].get("interruptible"):
-                resources.interruptible = self.provider_data["resources"]["interruptible"]
-            if self.provider_data["resources"].get("local"):
-                resources.local = self.provider_data["resources"]["local"]
-            if (
-                resources.cpus
-                or resources.memory_mib
-                or resources.gpus
-                or resources.shm_size_mib
-                or resources.interruptible
-                or resources.local
-            ):
-                return resources
+    def _resources(self) -> Requirements:
+        resources = Requirements()
+        cpu = self.provider_data["resources"].get("cpu", DEFAULT_CPU)
+        if not str(cpu).isnumeric():
+            sys.exit("resources.cpu should be an integer")
+        cpu = int(cpu)
+        if cpu > 0:
+            resources.cpus = cpu
+        memory = self.provider_data["resources"].get("memory", DEFAULT_MEM)
+        resources.memory_mib = _str_to_mib(memory)
+        gpu = self.provider_data["resources"].get("gpu")
+        if gpu:
+            if str(gpu).isnumeric():
+                gpu = int(self.provider_data["resources"]["gpu"])
+                if gpu > 0:
+                    resources.gpus = GpusRequirements(count=gpu)
             else:
-                return None
+                gpu_count = 0
+                gpu_name = None
+                if str(gpu.get("count")).isnumeric():
+                    gpu_count = int(gpu.get("count"))
+                if gpu.get("name"):
+                    gpu_name = gpu.get("name")
+                    if not gpu_count:
+                        gpu_count = 1
+                if gpu_count:
+                    resources.gpus = GpusRequirements(count=gpu_count, name=gpu_name)
+        for resource_name in self.provider_data["resources"]:
+            if resource_name.endswith("/gpu") and len(resource_name) > 4:
+                if not str(self.provider_data["resources"][resource_name]).isnumeric():
+                    sys.exit(f"resources.'{resource_name}' should be an integer")
+                gpu = int(self.provider_data["resources"][resource_name])
+                if gpu > 0:
+                    resources.gpus = GpusRequirements(count=gpu, name=resource_name[:-4])
+        if self.provider_data["resources"].get("shm_size"):
+            resources.shm_size_mib = _str_to_mib(self.provider_data["resources"]["shm_size"])
+        if self.provider_data["resources"].get("interruptible"):
+            resources.interruptible = self.provider_data["resources"]["interruptible"]
+        if self.provider_data["resources"].get("local"):
+            resources.local = self.provider_data["resources"]["local"]
+        return resources
 
     @staticmethod
     def _extend_commands_with_env(commands, env):
